@@ -1,5 +1,5 @@
-; ACL2 Version 4.1 -- A Computational Logic for Applicative Common Lisp
-; Copyright (C) 2010  University of Texas at Austin
+; ACL2 Version 4.2 -- A Computational Logic for Applicative Common Lisp
+; Copyright (C) 2011  University of Texas at Austin
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
 ; (C) 1997 Computational Logic, Inc.  See the documentation topic NOTE-2-0.
@@ -376,6 +376,8 @@
     'character)
    ((atom form)
     t)
+   ((eq (car form) 'return-last)
+    (output-type-for-declare-form-rec (cadr (last form)) flet-alist))
    ((assoc (car form) flet-alist :test 'eq)
     (cdr (assoc (car form) flet-alist :test 'eq)))
    ((member (car form) '(flet labels)
@@ -493,8 +495,8 @@
 ; We return a list of output types, one per value.  So if #-acl2-mv-as-values,
 ; then we always return a list of length one.
 
-  #-acl2-mv-as-values
-  (declare (ignore fn))
+  (when (eq fn 'return-last)
+    (return-from output-type-for-declare-form '*))
   #-acl2-mv-as-values
   (let* ((*acl2-output-type-abort* nil) ; protect for call on next line
          (result (output-type-for-declare-form-rec form nil)))
@@ -1104,25 +1106,27 @@ notation causes an error and (b) the use of ,. is not permitted."
 ; The following function used to be defined in axioms.lisp (with
 ; #-acl2-loop-only), but we need it here.
 
-(defconstant *main-lisp-package*
-  (find-package "COMMON-LISP"))
-
 (defun symbol-package-name (x)
+
+; Warning: This function assumes that x is not a bad-lisp-objectp.  In
+; particular, see the Invariant on Symbols in the Common Lisp Package,
+; discussed in a comment in bad-lisp-objectp, which allows us to assume that if
+; x resides in the "COMMON-LISP" package and does not have its
+; *initial-lisp-symbol-mark* property set, then its symbol-package is the
+; *main-lisp-package*.
+
   (cond ((get x *initial-lisp-symbol-mark*))
         ((let ((p (symbol-package x)))
+           (cond ((eq p *main-lisp-package*)
 
-; The following code should be kept in sync with the gcl-only code in the
-; vicinity of the calls of rename-package in acl2.lisp, and with the definition
-; of *main-lisp-package-name*.  The point is that the low-level package name
-; for Lisp symbols is "LISP" in GCL but we want to treat it as "COMMON-LISP" in
-; all Lisp implementations.  Thus, the extra test for the *main-lisp-package*
-; is probably not necessary except for GCL, but we go ahead for the sake of
-; uniformity and robustness.
+; We could just return *main-lisp-package-name-raw* in this case (but do not
+; skip this case, since in GCL, (package-name *main-lisp-package*) is "LISP",
+; not "COMMON-LISP" (which is what we need here).  But we go ahead and set
+; *initial-lisp-symbol-mark* in order to bypass this code next time.
 
-           (if (eq p *main-lisp-package*)
-               (setf (get x *initial-lisp-symbol-mark*)
-                     "COMMON-LISP")
-             (and p (package-name p)))))
+                  (setf (get x *initial-lisp-symbol-mark*)
+                        *main-lisp-package-name-raw*))
+                 (t (and p (package-name p))))))
 
 ; We use ERROR now because we cannot print symbols without packages
 ; with ACL2 functions.
@@ -1137,11 +1141,7 @@ notation causes an error and (b) the use of ,. is not permitted."
 (defmacro gv (fn args val)
   (sublis `((funny-fn . ,fn)
             (funny-args . ,args))
-          `(let ((gc-on
-                  (not (member (f-get-global 'guard-checking-on
-                                             *the-live-state*)
-                               '(nil :none)
-                               :test 'eq))))
+          `(let ((gc-on (not (gc-off *the-live-state*))))
              (if (or gc-on
                      (f-get-global 'safe-mode *the-live-state*))
                  (throw-raw-ev-fncall
@@ -1192,6 +1192,68 @@ notation causes an error and (b) the use of ,. is not permitted."
   #+apple (return-from get-os :apple)
   #-(or unix apple mswindows) nil)
 
+(defun our-truename (filename &optional namestringp)
+
+; For now, assume that namestringp is not supplied.
+
+; This function is intended to return nil if filename does not exist.  We thus
+; rely on the CL HyperSpec, where it says of truename that "An error of type
+; file-error is signaled if an appropriate file cannot be located within the
+; file system for the given filespec"; and we also rely on the Allegro CL
+; documentation for function pathname-resolve-symbolic-links, which says: "When
+; pathname names a non-existent pathname, an error is signaled...."
+
+; We return (ignore-errors (truename x)) instead of (probe-file x) because we
+; have seen CLISP 2.48 cause an error when probe-file is called on a directory
+; name.  Unfortunately, we can't do that with GCL 2.6.7, which doesn't have
+; ignore-errors.  Also unfortunately, Allegro CL 8.0 (also probably earlier
+; versions, and maybe later versions) does not fully resolve symbolic links
+; using truename, even with value T for keyword :follow-symlinks.
+
+; Finally, consider namestringp.  If nil, then as above we either return nil or
+; the truename (a pathname object).  Otherwise, we return the namestring of
+; such a truename, with the following treatment if that truename is nil: return
+; nil if namestringp is :safe, else cause an error.
+
+  (let ((truename
+         (cond
+          #+allegro
+          ((fboundp 'excl::pathname-resolve-symbolic-links)
+           (ignore-errors
+            (qfuncall excl::pathname-resolve-symbolic-links
+                      filename)))
+          #+(and gcl (not cltl2))
+          ((fboundp 'si::stat) ; try to avoid some errors
+           (and (or (qfuncall si::stat filename)
+
+; But filename might be a directory, in which case the si::stat call above
+; could return nil; so we try again.
+
+                    (and (or (equal filename "")
+                             (not (eql (char filename (1- (length filename)))
+                                       #\/)))
+                         (qfuncall si::stat
+                                   (concatenate 'string filename "/"))))
+                (truename filename)))
+          #+(and gcl (not cltl2))
+          (t (truename filename))
+          #-(and gcl (not cltl2))
+          (t
+
+; Here we also catch the case of #+allegro if
+; excl::pathname-resolve-symbolic-links is not defined.
+
+           (ignore-errors (truename filename))))))
+    (cond ((null namestringp)
+           truename)
+          ((null truename)
+           (cond ((eq namestringp :safe) nil)
+                 (t (qfuncall
+                     interface-er
+                     "Unable to obtain the truename of file ~x0."
+                     filename))))
+          (t (namestring truename)))))
+
 (defun our-pwd ()
 
 ; Warning: Do not be tempted to use (getenv$-raw "PWD").  The PWD environment
@@ -1199,7 +1261,7 @@ notation causes an error and (b) the use of ,. is not permitted."
 ; make invokes another make in a different directory.
 
   (qfuncall pathname-os-to-unix
-            (namestring (truename ""))
+            (our-truename "" t)
             (get-os)
             *the-live-state*))
 
