@@ -1,5 +1,5 @@
-; ACL2 Version 4.1 -- A Computational Logic for Applicative Common Lisp
-; Copyright (C) 2010  University of Texas at Austin
+; ACL2 Version 4.2 -- A Computational Logic for Applicative Common Lisp
+; Copyright (C) 2011  University of Texas at Austin
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
 ; (C) 1997 Computational Logic, Inc.  See the documentation topic NOTE-2-0.
@@ -247,8 +247,8 @@ implementations.")
 
          (setq source-directory
                (subseq source-directory 0 (1- (length source-directory))))))
-  (cond ((not (equal (namestring (truename (format nil "~a/" source-directory)))
-                     (namestring (truename ""))))
+  (cond ((not (equal (our-truename (format nil "~a/" source-directory) :safe)
+                     (our-truename "" t)))
          (error "We expected to be in the directory~%~s~%~
                  but instead are apparently in the directory~%~s .~%~
                  Either issue, in Unix, the command~%~
@@ -257,7 +257,7 @@ implementations.")
                  which the function COPY-DISTRIBUTION was called,~%~
                  in order to give it the correct second argument."
                 source-directory
-                (namestring (truename ""))
+                (our-truename "" t)
                 source-directory)))
 
 ; Next, check that everything exists that is supposed to.
@@ -425,17 +425,48 @@ implementations.")
 (defvar *saved-build-date-lst*)
 (defvar *saved-mode*)
 
+(defconstant *acl2-svn-revision-string*
+   "~% WARNING: Do not redistribute.  This is NOT an ACL2 release; it is,~
+    ~% rather, an svn distribution, ~a.~
+    ~% The authors of ACL2 consider svn distributions to be experimental.~%")
+
+(defun acl2-svn-revision-string ()
+
+; Put symbol :release into file acl2-startup-info.txt before doing a release.
+; Otherwise, we expect that file to start with a string of the form
+; "$Revision: n $".
+
+  (let ((file "acl2-startup-info.txt"))
+    (cond ((probe-file file)
+           (let ((val (with-open-file
+                       (str file :direction :input)
+                       (read str))))
+             (cond ((eq val :release)
+                    "")
+                   ((stringp val)
+                    (format nil *acl2-svn-revision-string* val))
+                   (t (error "Illegal value in file ~s: ~s"
+                             file val)))))
+          (t (error "File ~s appears not to exist." file)))))
+
 (defvar *saved-string*
   (concatenate
    'string
    "~% ~a built ~a.~
-   ~% Copyright (C) 2010  University of Texas at Austin~
-   ~% ACL2 comes with ABSOLUTELY NO WARRANTY.  This is free software and you~
-   ~% are welcome to redistribute it under certain conditions.  For details,~
-   ~% see the GNU General Public License.~%~
-    ~a"
+    ~% Copyright (C) 2011  University of Texas at Austin"
+   "~% ACL2 comes with ABSOLUTELY NO WARRANTY.  This is free software and you~
+    ~% are welcome to redistribute it under certain conditions.  For details,~
+    ~% see the GNU General Public License.~%"
+
+; The following is empty if acl2-startup-info.txt begins with the symbol
+; :release.
+
+   (acl2-svn-revision-string)
+
+   "~a"
    #+hons
-   "~%~% Experimental modification for HONS, memoization, and applicative hash tables.~%"
+   "~%~% Experimental modification for HONS, memoization, and applicative hash~
+    ~% tables.~%"
    #+acl2-par
    "~%~% Experimental modification for parallel evaluation.  Please expect at~
     ~% most limited maintenance for this version~%"
@@ -1221,7 +1252,7 @@ implementations.")
                       ("~a~%"
                        (let ((default-dir
                                (or (ccl::getenv "CCL_DEFAULT_DIRECTORY")
-                                   (let ((path (truename "ccl:")))
+                                   (let ((path (our-truename "ccl:")))
                                      (and path
                                           (qfuncall pathname-os-to-unix
                                                     (namestring path)
