@@ -1,5 +1,5 @@
-; ACL2 Version 4.1 -- A Computational Logic for Applicative Common Lisp
-; Copyright (C) 2010  University of Texas at Austin
+; ACL2 Version 4.2 -- A Computational Logic for Applicative Common Lisp
+; Copyright (C) 2011  University of Texas at Austin
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
 ; (C) 1997 Computational Logic, Inc.  See the documentation topic
@@ -234,16 +234,15 @@
 ;                               FILES
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-; This is the file acl2.lisp, the first source file for ACL2.  The
-; names of the other ACL2 source files are listed under *acl2-files*.
+; This is the file acl2.lisp, the first source file for ACL2.  The names of the
+; other ACL2 source files are listed under *acl2-files*.
 
-; All of ACL2 is written in Common Lisp and we expect that ACL2 will
-; run in any Common Lisp except for those Common Lisps which fail the
-; tests we execute upon loading this file, acl2.lisp.  With the
-; exception of this and other initialiation files, files *-raw.lisp,
-; and those constructs after the #-acl2-loop-only passim only
-; (especially in the file axioms.lisp), ACL2 is written in the
-; applicative Common Lisp for which ACL2 is a verification system.
+; All of ACL2 is written in Common Lisp and we expect that ACL2 will run in any
+; Common Lisp except for those Common Lisps which fail the tests we execute
+; upon loading this file, acl2.lisp.  With the exception of this and other
+; initialiation files, files *-raw.lisp, and those constructs after the
+; #-acl2-loop-only readtime conditional, ACL2 is written in the applicative
+; Common Lisp for which ACL2 is a verification system.
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;                          LISP BUGS AND QUIRKS
@@ -538,11 +537,20 @@
 
 ; The order of these files determines compilation order.
 
+; Note regarding backups at UT CS:
+
+; Even though it's convenient to refer to our UT CS development directory as
+; /projects/acl2/devel/, we'll need to get backups from
+; /v/filer4b/v11q002/acl2space/acl2/.snapshot/*/devel, not from
+; /projects/acl2/.snapshot/*/devel.  The latter is just a soft link to
+; /projects/acl2/devel, i.e., to /v/filer4b/v11q002/acl2space/acl2/devel.
+
   '(
     #+acl2-par "multi-threading-raw"
     "axioms"
     "memoize" ; but only get special under-the-hood treatment with #+hons
     "hons"    ; but only get special under-the-hood treatment with #+hons
+    "boot-strap-pass-2"
     "basis"
     "parallel" ; but only get special under-the-hood treatment with #+acl2-par
     #+acl2-par "parallel-raw"
@@ -626,7 +634,7 @@ ACL2 from scratch.")
    (setq acl2::*copy-of-acl2-version*
 ;  Keep this in sync with the value of acl2-version in *initial-global-table*.
          (concatenate 'string
-                      "ACL2 Version 4.1"
+                      "ACL2 Version 4.2"
                       #+non-standard-analysis
                       "(r)"
                       #+(and mcl (not ccl))
@@ -716,15 +724,22 @@ ACL2 from scratch.")
 
 ; Next we define the initial global and *1* packages.
 
+(defconstant *main-lisp-package*
+  (find-package "COMMON-LISP"))
+
+(defconstant *main-lisp-package-name-raw*
+; Keep this in sync with *main-lisp-package-name*.
+  "COMMON-LISP")
+
 ; Keep the following in sync with *main-lisp-package-name*.
-(make-global-package "COMMON-LISP")
+(make-global-package *main-lisp-package-name-raw*)
 (make-global-package "KEYWORD")
 (make-global-package "ACL2")
 (make-global-package "ACL2-INPUT-CHANNEL")
 (make-global-package "ACL2-OUTPUT-CHANNEL")
 
 ; Keep the following in sync with *main-lisp-package-name*.
-(make-*1*-package "COMMON-LISP")
+(make-*1*-package *main-lisp-package-name-raw*)
 ; Functions cannot be defined in the keyword package, so we do not do so.
 ;  (make-*1*-package "KEYWORD")
 (make-*1*-package "ACL2")
@@ -738,8 +753,13 @@ ACL2 from scratch.")
 (defparameter acl2::*initial-lisp-symbol-mark*
   'acl2_invisible::initial-lisp-symbol-mark)
 
-(dolist (sym *copy-of-common-lisp-symbols-from-main-lisp-package*)
-        (setf (get sym *initial-lisp-symbol-mark*) "COMMON-LISP"))
+(let ((cl-pkg-name *main-lisp-package-name-raw*))
+  (dolist (sym *copy-of-common-lisp-symbols-from-main-lisp-package*)
+
+; We support the Invariant on Symbols in the Common Lisp Package.  See the long
+; comment about this invariant in the symbolp case of bad-lisp-objectp.
+
+    (setf (get sym *initial-lisp-symbol-mark*) cl-pkg-name)))
 
 (defconstant *acl2-package* (find-package "ACL2"))
 
@@ -993,6 +1013,11 @@ ACL2 from scratch.")
 #. to cause evaluation during READing (b) and to define our own version
 of backquote.")
 
+(defparameter *host-readtable*
+  (copy-readtable)
+  "*host-readtable* is the original readtable provided by the host Lisp,
+which is saved just in case it's needed later.")
+
 (defun define-sharp-dot ()
   (set-dispatch-macro-character
    #\#
@@ -1599,16 +1624,6 @@ Newline
       (set 'state *the-live-state*)
       "ACL2"))))
 
-(defun load-acl2-execution-environment ()
-  (let ((*readtable* *acl2-readtable*))
-    (load-compiled (make-pathname :name "axioms"
-                                  :type (if *suppress-compile-build-time*
-                                            *lisp-extension*
-                                          *compiled-file-extension*)))
-    (load "defpkgs.lisp")
-    (in-package "ACL2")
-    "ACL2"))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;                            DECLARATIONS
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -1626,15 +1641,28 @@ Newline
 
 (defun exit-lisp (&optional (status '0 status-p))
 
-; The status will be returned as the exit status (shell variable $?).
-; We avoid passing the status argument when it is 0, in case Windows or other
-; operating systems get confused by it.
+; The status (an integer) will be returned as the exit status (shell variable
+; $?).  We avoid passing the status argument when it is 0, in case Windows or
+; other operating systems get confused by it.  However, this function is a
+; no-op if we are already in the process of quitting via this function; see the
+; comment below the occurrence below of *acl2-panic-exit-status*.
 
 ; It appeared at one point that (ccl::quit 0) is more reliable than
 ; (ccl::quit), but that's no longer clear.  Still, it seems reasonable to pass
 ; the status explicitly to the individual Lisp's exit function if that status
 ; is passed explicitly here -- hence the use of status-p.
 
+  (cond
+   ((null status) ; shouldn't happen
+    (error "Passed null status to exit-lisp!"))
+   (*acl2-panic-exit-status*
+
+; We have seen various type errors and bus errors when attempting to quit in
+; CCL.  Gary Byers pointed out that this may be caused by attempting to quit
+; while in the process of quitting.  So, we avoid doing a quit if already in
+; the process of quitting.
+
+    (return-from exit-lisp nil)))
   (setq *acl2-panic-exit-status* status)
   #+clisp
   (if status-p (user::exit status) (user::exit))
