@@ -1,3 +1,19 @@
+; Arithmetic-5 Library
+; Copyright (C) 2009 Robert Krug <rkrug@cs.utexas.edu>
+;
+; This program is free software; you can redistribute it and/or modify it under
+; the terms of the GNU General Public License as published by the Free Software
+; Foundation; either version 2 of the License, or (at your option) any later
+; version.
+;
+; This program is distributed in the hope that it will be useful but WITHOUT
+; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+; details.
+;
+; You should have received a copy of the GNU General Public License along with
+; this program; if not, write to the Free Software Foundation, Inc., 51
+; Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -287,14 +303,12 @@
       (let ((rule (select-instantiated-elim-rule-hint cl type-alist eliminables
 						      fns-to-elim ens wrld)))
         (cond ((null rule) (mv (list cl) nil nil))
-              (t (mv-let (contradictionp new-clause elim-vars1 ele)
+              (t (mv-let (new-clause elim-vars1 ele)
                    (apply-instantiated-elim-rule rule cl type-alist
                                                  avoid-vars ens wrld)
-                   (let ((clauses1 (if contradictionp
-                                       nil
-                                     (split-on-assumptions
-                                      (access elim-rule rule :hyps)
-                                      cl nil))))
+                   (let ((clauses1 (split-on-assumptions
+                                    (access elim-rule rule :hyps)
+                                    cl nil)))
                      (cond
                       ((equal new-clause *true-clause*)
                        (mv clauses1 elim-vars1 (list ele)))
@@ -330,14 +344,12 @@
 		     ;; On with the original show.
 		     ;; We are assuming that any choices to be made occur
 		     ;; at the very beginning.
-		     (mv-let (contradictionp new-clause elim-vars1 ele)
+		     (mv-let (new-clause elim-vars1 ele)
 		       (apply-instantiated-elim-rule rule cl type-alist
 						     avoid-vars ens wrld)
-		       (let ((clauses1 (if contradictionp
-					   nil
-					 (split-on-assumptions
-					  (access elim-rule rule :hyps)
-					  cl nil))))
+		       (let ((clauses1 (split-on-assumptions
+                                        (access elim-rule rule :hyps)
+                                        cl nil)))
 			 (cond
 			  ((equal new-clause *true-clause*)
 			   (mv clauses1 elim-vars1 (list ele)))
