@@ -157,11 +157,14 @@ sub get_cert_time {
 
 # Given a .cert file, gets the total user + system time recorded in the
 # corresponding .time file.  If not found, prints a warning and returns 0.
+# Given an .acl2x file, gets the time recorded in the corresponding
+# .acl2x.time file.
 
     my $path = shift;
     my $warnings = shift;
 
     $path =~ s/\.cert$/\.time/;
+    $path =~ s/\.acl2x$/\.acl2x\.time/;
     
     if (open (my $timefile, "<", $path)) {
 	while (my $the_line = <$timefile>) {
@@ -217,7 +220,7 @@ sub make_costs_table_aux {
 
     if ($certdeps) {
 	foreach my $dep (@{$certdeps}) {
-	    if ($dep =~ /\.cert$/) {
+	    if ($dep =~ /\.(cert|acl2x)$/) {
 		my $this_dep_costs = make_costs_table_aux($dep, $deps, $costs, $warnings, $short);
 		# check for dependency loop:
 		if ($this_dep_costs) {
@@ -641,7 +644,7 @@ sub scan_ld {
 	}
 	close($ld);
     } else {
-	print "Warning: Could not open $fname: $!\n";
+	print "Warning: scan_ld: Could not open $fname: $!\n";
     }
 }
 
@@ -667,67 +670,41 @@ sub scan_book {
 	    }
 	    close($lisp);
 	} else {
-	    print "Warning: Could not open $fname: $!\n";
+	    print "Warning: scan_book: Could not open $fname: $!\n";
 	}
     }
 }
     
+sub scan_two_pass {
+    my $fname = shift;
 
+    print "scan_two_pass $fname\n" if $debugging;
+
+    if ($fname) {
+	# Scan the file for ";; two-pass certification"
+	if (open(my $file, "<", $fname)) {
+	    my $regexp = ";; two-pass certification";
+	    while (my $the_line = <$file>) {
+		my $match = $the_line =~ m/$regexp/;
+		if ($match) {
+		    print "two pass: $fname\n" if $debugging;
+		    return 1;
+		}
+	    }
+	    close($file);
+	}
+    }
+    return 0;
+}
+		
     
-    
-
-# During a dependency search, this is run with $target set to each
-# cert and source file in the dependencies of the top-level targets.
-# If the target has been seen before, then it returns immediately.
-sub add_deps {
-    my $target = shift;
-    my $seen = shift;
-    my $sources = shift;
-
-
-    if (exists $seen->{$target}) {
-	# We've already calculated this file's dependencies.
-	return;
-    }
-
-    if ($target !~ /\.cert$/) {
-	push(@{$sources}, $target);
-	$seen->{$target} = 0;
-	return;
-    }
-
-    if (excludep($target)) {
-	$seen->{$target} = 0;
-	return;
-    }
-
-    print "add_deps $target\n" if $debugging;
-
-    my $local_dirs = {};
-    my $base = $target;
-    $base =~ s/\.cert$//;
-    my $pfile = $base . ".p";
+# Find dependencies o
+sub find_deps {
+    my $base = shift;
     my $lispfile = $base . ".lisp";
 
-    # Clean the cert and out files if we're cleaning.
-    if ($clean_certs) {
-	my $outfile = $base . ".out";
-	my $timefile = $base . ".time";
-	my $compfile = $base . ".lx64fsl";
-	unlink($target) if (-e $target);
-	unlink($outfile) if (-e $outfile);
-	unlink($timefile) if (-e $timefile);
-	unlink($compfile) if (-e $compfile);
-    }
-
-    # First check that the corresponding .lisp file exists.
-    if (! -e $lispfile) {
-	print "Error: Need $lispfile to build $target.\n";
-	return;
-    }
-
-    $seen->{$target} = [ $lispfile ];
-    my $deps = $seen->{$target};
+    my $deps = [ $lispfile ];
+    my $local_dirs = {};
 
     # If a corresponding .acl2 file exists or otherwise if a
     # cert.acl2 file exists in the directory, we need to scan that for dependencies as well.
@@ -773,10 +750,74 @@ sub add_deps {
 	    }
 	    close $im;
 	} else {
-	    print "Warning: Could not open $imagefile: $!\n";
+	    print "Warning: find_deps: Could not open image file $imagefile: $!\n";
 	}
     }
 
+    return $deps;
+
+}    
+    
+
+# During a dependency search, this is run with $target set to each
+# cert and source file in the dependencies of the top-level targets.
+# If the target has been seen before, then it returns immediately.
+sub add_deps {
+    my $target = shift;
+    my $seen = shift;
+    my $sources = shift;
+
+
+    if (exists $seen->{$target}) {
+	# We've already calculated this file's dependencies.
+	return;
+    }
+
+    if ($target !~ /\.cert$/) {
+	push(@{$sources}, $target);
+	$seen->{$target} = 0;
+	return;
+    }
+
+    if (excludep($target)) {
+	$seen->{$target} = 0;
+	return;
+    }
+
+    print "add_deps $target\n" if $debugging;
+
+    my $local_dirs = {};
+    my $base = $target;
+    $base =~ s/\.cert$//;
+    my $lispfile = $base . ".lisp";
+
+    # Clean the cert and out files if we're cleaning.
+    if ($clean_certs) {
+	my $outfile = $base . ".out";
+	my $timefile = $base . ".time";
+	my $compfile = $base . ".lx64fsl";
+	unlink($target) if (-e $target);
+	unlink($outfile) if (-e $outfile);
+	unlink($timefile) if (-e $timefile);
+	unlink($compfile) if (-e $compfile);
+    }
+
+    # First check that the corresponding .lisp file exists.
+    if (! -e $lispfile) {
+	print "Error: Need $lispfile to build $target.\n";
+	return;
+    }
+
+    my $deps = find_deps($base);
+
+    if (scan_two_pass($lispfile)) {
+	my $acl2xfile = $base . ".acl2x";
+	$seen->{$target} = [ $acl2xfile ];
+	$seen->{$acl2xfile} = $deps;
+    } else {
+	$seen->{$target} = $deps;
+    }
+
     if ($print_deps) {
 	print "Dependencies for $target:\n";
 	foreach my $dep (@{$deps}) {
