@@ -335,7 +335,7 @@
  ()
  (define-trusted-clause-processor
    strengthen-cl-program2
-   nil
+   (f0)
    :partial-theory (encapsulate ((f0 (x) t))
                                 (local (defun f0 (x) x))
                                 (defthm f0-prop
@@ -430,17 +430,17 @@
                        cl)
                  (list term)))))
 
-(must-fail
-; F0, supporting g0, has unknown constraints from dependent clause-processor
-; strengthen-cl-program2.
- (defthm correctness-of-strengthen-cl-b
-   (implies (and (pseudo-term-listp cl)
-                 (alistp a)
-                 (evl (conjoin-clauses
-                       (clauses-result (strengthen-cl-b cl term state)))
-                      a))
-            (evl (disjoin cl) a))
-   :rule-classes :clause-processor))
+; F0, supporting g0, is constrained by strengthen-cl-b, which has unknown
+; constraints from dependent clause-processor strengthen-cl-program2.  However,
+; we do know that f0 is the only direct supporter of strengthen-cl-b.
+(defthm correctness-of-strengthen-cl-b
+  (implies (and (pseudo-term-listp cl)
+                (alistp a)
+                (evl (conjoin-clauses
+                      (clauses-result (strengthen-cl-b cl term state)))
+                     a))
+           (evl (disjoin cl) a))
+  :rule-classes :clause-processor)
 
 (defun strengthen-cl-c (cl term state)
   (declare (xargs :stobjs state :guard (consp term)))
@@ -898,9 +898,12 @@
    :hints (("Goal" :clause-processor (trivial-cl-proc cl)))
    :rule-classes nil))
 
-(must-fail ; unknown constraints in defaxiom
- (defaxiom bad
-   (true-listp (strengthen-cl-b cl term state))))
+; Unknown constraints in a defaxiom were prohibited through Version 3.6.1 of
+; ACL2.  However, now that we use the supporters of a dependent
+; clause-processor to determine ancestors, there is no longer that problem.
+(defaxiom formerly-bad
+  (true-listp (strengthen-cl-b cl term state))
+  :rule-classes nil)
 
 (must-fail ; :partial-theory must be an encapsulate
  (encapsulate ; hard error thrown below defeats must-fail without encapsulate
@@ -911,24 +914,25 @@
     :partial-theory (car nil)
     :ttag my-ttag)))
 
-; Non-executable clause-processors are illegal.
+; Non-executable clause-processors were illegal, but that is no longer the case
+; starting with v4-0 because we want to support attachments.
 
-(defun trivial-cl-proc-nonexec (cl)
-  (declare (xargs :non-executable t))
+(defun-nx trivial-cl-proc-nonexec (cl)
   (list cl))
 
-(must-fail
- (defthm correctness-of-trivial-cl-proc-nonexec
-   (implies (and (pseudo-term-listp cl)
-                 (alistp a)
-                 (ev3 (conjoin-clauses
-                       (trivial-cl-proc-nonexec cl))
-                      a))
-            (ev3 (disjoin cl) a))
-   :rule-classes :clause-processor))
+(encapsulate
+ ()
+ (local
+  (defthm correctness-of-trivial-cl-proc-nonexec
+    (implies (and (pseudo-term-listp cl)
+                  (alistp a)
+                  (ev3 (conjoin-clauses
+                        (trivial-cl-proc-nonexec cl))
+                       a))
+             (ev3 (disjoin cl) a))
+    :rule-classes :clause-processor)))
 
-(defun strengthen-cl-nonexec (cl term)
-  (declare (xargs :non-executable t))
+(defun-nx strengthen-cl-nonexec (cl term)
 ; sad that we can't translate term first
   (cond ((pseudo-termp term) ; sad that we can't use termp!
          (list (cons (list 'not term)
@@ -943,11 +947,13 @@
                      `(translate ',term t t t 'top-level (w state) state))
                  (list cl)))))
 
-(must-fail ; non-executable
- (define-trusted-clause-processor
-   strengthen-cl-nonexec
-   nil
-   :ttag my-ttag))
+(encapsulate
+ ()
+ (local ; non-executable is OK starting with v4-0
+  (define-trusted-clause-processor
+    strengthen-cl-nonexec
+    nil
+    :ttag my-ttag)))
 
 ; Now verify the original (trusted) clause processor.
 
