@@ -65,7 +65,7 @@
 
 (defmacro ansfl-list (l x)
 
-; [Jared]: BOZO please document this.  It's used.
+; (ansfl-list (a b c ...) x) -- frees a, b, c, ..., returns x
 
   (if (atom l)
       x
@@ -76,6 +76,12 @@
 (defn ansfl-last-list (r bindings)
 
 ; [Jared]: BOZO please document this.  It's used in het*.
+
+; bindings is an alist.  in het* the bindings are names being bound
+; like in a let*.
+;
+; all of the names being bound are freed, then we return r.
+
   (if (atom bindings)
       r
     `(ansfl ,(ansfl-last-list r (gentle-cdr bindings))
@@ -83,26 +89,29 @@
 
 (defmacro het* (bindings &rest r)
 
-; [Jared]: BOZO please document this.
-
 ; This implementation of het* is somewhat defective in that it is
 ; incapable of returning multiple values.  We cannot see how to fix
 ; it.
 
+; this is basically let*, but we try to fast-alist-free everything that gets
+; bound.  which works out, in a weird kind of way, for anything that
+; isn't a fast alist anyway, but is really pretty gross.
+
   `(let* ,bindings
      ,@(butlast r 1)
      ,(ansfl-last-list (car (last r)) bindings)))
 
 (defmacro with-fast-list (var term name form)
+
+; bind a variable to a fast-alist created by binding every element of term to t,
+; with the final name name.  then run form and free var.
+
   `(let ((,var (hons-put-list
                 ,term
                 t
                 ,name)))
      (ansfl ,form ,var)))
 
-(defmacro with-fast-alist (var l1 l2 name form)
-  `(let ((,var (hons-put-list ,l1 ,l2 ,name)))
-     (ansfl ,form ,var)))
 
 (defn hons-put-list (keys values l)
 
@@ -234,6 +243,17 @@
 
   (hons-dups-p1 l '*hons-dups-p*))
 
+(defn hons-duplicates1 (l tab)
+  (cond ((atom l) (ansfl nil tab))
+        ((hons-get (car l) tab)
+         (cons (car l) (hons-duplicates1 (cdr l) tab)))
+        (t (hons-duplicates1 (cdr l) (hons-acons (car l) t tab)))))
+
+(defn hons-duplicates (l)
+  (hons-duplicates1 l nil))
+         
+
+
 (defn hons-remove-duplicates1 (l tab)
   (cond ((atom l)
          (ansfl nil tab))
@@ -445,12 +465,12 @@
          (let ((len1 (len l1))
                (len2 (len l2)))
            (cond ((and (>= len2 len1)
-                       (>= len2 *magic-number-for-hashing*))
+                       (>= len1 *magic-number-for-hashing*))
                   (with-fast-list
                    fl2 l2 '*hons-union*
                    (hons-union1 l1 fl2 l2)))
                  ((and (>= len1 len2)
-                       (>= len1 *magic-number-for-hashing*))
+                       (>= len2 *magic-number-for-hashing*))
                   (with-fast-list
                    fl1 l1 '*hons-union*
                    (hons-union1 l2 fl1 l1)))
@@ -940,3 +960,13 @@
 ;;          (hons-remove-equal-cons x (cdr y)))
 ;;         (t (cons (car y) (hons-remove-equal-cons x (cdr y))))))
 
+
+
+
+
+;; [Jared and Sol] deleting this because it's never used and we have a new fancy
+;; thing called with-fast-alist that is better
+
+;; (defmacro with-fast-alist (var l1 l2 name form)
+;;   `(let ((,var (hons-put-list ,l1 ,l2 ,name)))
+;;      (ansfl ,form ,var)))
