@@ -413,7 +413,7 @@
                 (< 0 Q))
            (EXACTP (DDECODE X P Q)
                    (+ (* -1 (BIAS Q))
-                      (EXPT 2 (+ -1 Q))
+                      (EXPT 2 (1- Q))
                       (EXPO (ISIGF X P)))))
   :hints (("Goal" :in-theory (enable ddecode dencodingp bias)))))     
 
@@ -510,8 +510,8 @@
                  (INTEGERP P)
                  (<= 1 P)
                  )
-            (integerp (+ (* -1 (EXPT 2 (+ -1 P)))
-                         (* (SIG X) (EXPT 2 (+ -1 P))))
+            (integerp (+ (* -1 (EXPT 2 (1- P)))
+                         (* (SIG X) (EXPT 2 (1- P))))
                       ))
    :hints (("Goal" :in-theory (enable exactp)
             :use ( sig-lower-bound)))))
@@ -561,7 +561,7 @@
                  (INTEGERP Q)
                  (< 0 Q))
             (< (IEXPOF (NENCODE X P Q) P Q)
-               (+ -1 (EXPT 2 Q))))
+               (1- (EXPT 2 Q))))
    :hints (("Goal"  :in-theory (e/d (nrepp nencode iexpof bvecp
                                        bits-tail) (sig-lower-bound sig-upper-bound))
             :use (sig-upper-bound sig-lower-bound)))))
@@ -584,8 +584,8 @@
                  (integerp q)
                  (> q 0))
             (equal (* (SIG X)
-                      (EXPT 2 (+ -3 P (EXPO X) (EXPT 2 (+ -1 Q)))))
-                   (* (abs x) (EXPT 2 (+ -3 P (EXPT 2 (+ -1 Q)))))))
+                      (EXPT 2 (+ -3 P (EXPO X) (EXPT 2 (1- Q)))))
+                   (* (abs x) (EXPT 2 (+ -3 P (EXPT 2 (1- Q)))))))
    :hints (("Goal" :in-theory (set-difference-theories
                                (enable drepp sgn)
                                '(fp-rep-collapse abs))
@@ -603,7 +603,7 @@
                  (integerp q)
                  (> q 0))
             (bvecp (* (SIG X)
-                      (EXPT 2 (+ -3 P (EXPO X) (EXPT 2 (+ -1 Q)))))
+                      (EXPT 2 (+ -3 P (EXPO X) (EXPT 2 (1- Q)))))
                    (- p 1)))
            
    :hints (("Goal" :in-theory (set-difference-theories (enable dencodingp drepp dencode
@@ -614,7 +614,7 @@
             :use (dencodingp-dencode-hack-3
                   sig-upper-bound 
                   (:instance expt-weak-monotone
-                             (n (+ -3 P (EXPO X) (EXPT 2 (+ -1 Q))))
+                             (n (+ -3 P (EXPO X) (EXPT 2 (1- Q))))
                              (m (- p 2))))))))
 
 (defthm dencodingp-dencode
@@ -677,8 +677,8 @@
                 (integerp p)
                 (< 1 p)
                 )
-           (bvecp (+ (* -1 (expt 2 (+ -1 p)))
-                     (* (sig x) (expt 2 (+ -1 p))))
+           (bvecp (+ (* -1 (expt 2 (1- p)))
+                     (* (sig x) (expt 2 (1- p))))
                   (- p 1)))
   :hints (("goal" :in-theory (set-difference-theories
                               (enable bvecp)
@@ -726,7 +726,7 @@
                  (:instance bitn-bvecp-0 
                             (m q)
                             (x (* (SIG X)
-                                  (EXPT 2 (+ -3 P (EXPO X) (EXPT 2 (+ -1 Q))))))
+                                  (EXPT 2 (+ -3 P (EXPO X) (EXPT 2 (1- Q))))))
                             (n (- p 1)))))))
 
 (defthm isgnf-iencode
