@@ -1419,3 +1419,61 @@
 ;			(:instance expt-pos (x (- (expo x) n)))
                         ))))
 
+
+(local (include-book "../arithmetic/top"))
+
+; The next two lemmas are copied from near-proofs.lisp.
+
+(defthm plus-near-1
+  (implies (and (exactp x (+ k (- (expo x) (expo y))))
+                (rationalp x)
+                (>= x 0)
+                (rationalp y)
+                (>= y 0)
+                (integerp k)
+                )
+           (= (re (* (expt 2 (1- k)) (sig y)))
+              (re (* (expt 2 (1- (+ k (- (expo (+ x y)) (expo y)))))
+                     (sig (+ x y))))))
+  :rule-classes nil
+  :hints (("Goal" :in-theory (enable re sig exactp expt-split expt-minus))))
+
+(defthm plus-near-2
+  (implies (and (rationalp x)
+                (>= x 0)
+                (rationalp y)
+                (>= y 0)
+                (integerp k)
+                (exactp x (+ -1 k (- (expo x) (expo y)))))
+           (iff (evenp (fl (* (expt 2 (1- k)) (sig y))))
+                (evenp (fl (* (expt 2
+                                    (1- (+ k (- (expo (+ x y)) (expo y)))))
+                              (sig (+ x y)))))))
+  :otf-flg t
+  :rule-classes nil 
+  :hints (("Goal" :in-theory (e/d (expt-split
+                                   expt-minus
+                                   exactp sig ; EXPT-SPLIT-leading-constant
+                                   evenp ;this is sort of cheating...
+                                   ) ())
+           :use ((:instance exactp2 (n (+ k (expo x) (- (expo y)))))
+                 (:instance exactp-<=
+                            (m (+ -1 k (expo x) (- (expo y))))
+                            (n (+ k (expo x) (- (expo y)))))))))
+
+(defthm plus-near+
+  (implies (and (rationalp x)
+                (>= x 0)
+                (rationalp y)
+                (>= y 0)
+                (integerp k)
+                (exactp x (+ -1 k (- (expo x) (expo y)))))
+           (= (+ x (near+ y k))
+              (near+ (+ x y)
+                     (+ k (- (expo (+ x y)) (expo y))))))
+  :hints (("Goal" :in-theory (enable near+)
+           :use (plus-trunc plus-away plus-near-1 plus-near-2
+                            (:instance exactp-<=
+                                       (m (+ -1 k (expo x) (* -1 (expo y))))
+                                       (n (+ k (expo x) (* -1 (expo y))))))))
+  :rule-classes nil)
