@@ -12,10 +12,11 @@ what should lior0 of non-ints be?
 |#
 
 
-(defun natp (x)
-  (declare (xargs :guard t))
-  (and (integerp x)
-       (<= 0 x)))
+(local ; ACL2 primitive
+ (defun natp (x)
+   (declare (xargs :guard t))
+   (and (integerp x)
+        (<= 0 x))))
 
 (defund fl (x)
   (declare (xargs :guard (real/rationalp x)))
