@@ -27,10 +27,11 @@
                   0
                 (logand (ash x (- j)) (1- (ash 1 (1+ (- i j))))))))
 
-(defun natp (x)
-  (declare (xargs :guard t))
-  (and (integerp x)
-       (<= 0 x)))
+(local ; ACL2 primitive
+ (defun natp (x)
+   (declare (xargs :guard t))
+   (and (integerp x)
+        (<= 0 x))))
 
 (defund bvecp (x k)
   (declare (xargs :guard (integerp k)))
