@@ -1,5 +1,5 @@
-; ACL2 Version 4.1 -- A Computational Logic for Applicative Common Lisp
-; Copyright (C) 2010  University of Texas at Austin
+; ACL2 Version 4.2 -- A Computational Logic for Applicative Common Lisp
+; Copyright (C) 2011  University of Texas at Austin
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
 ; (C) 1997 Computational Logic, Inc.  See the documentation topic
@@ -135,6 +135,7 @@ is often slower than just using ~ilc[hons] directly when constructing the
 object.~/"
 
   ;; Has an under-the-hood implementation
+  (declare (xargs :mode :logic)) ; for attaching early to acl2x-expansion-alist
   x)
 
 #+(or acl2-loop-only (not hons))
@@ -851,94 +852,5 @@ Deprecated.  Alias for ~ilc[fast-alist-free]~/~/~/"
             (:executable-counterpart flush-hons-get-hash-table-link)
             ))
 
-; End of the proper hons code.
-;
-; The remainder of this file doesn't really belong here, but is just due to
-; "hons.lisp" coming last in the *acl2-pass-2-files*
-
 ; For some additional helper functions and lemmas, see the files
 ; books/misc/hons-help.lisp and books/misc/hons-help2.lisp.
-
-(deftheory definition-minimal-theory
-  (definition-runes
-    *definition-minimal-theory*
-    nil
-    world))
-
-(deftheory executable-counterpart-minimal-theory
-  (definition-runes
-    *built-in-executable-counterparts*
-    t
-    world))
-
-(deftheory minimal-theory
-
-; Warning: The resulting value must be a runic-theoryp.  See
-; theory-fn-callp.
-
-; Keep this definition in sync with translate-in-theory-hint.
-
-  (union-theories (theory 'definition-minimal-theory)
-                  (union-theories
-
-; Without the :executable-counterpart of force, the use of (theory
-; 'minimal-theory) will produce the warning "Forcing has transitioned
-; from enabled to disabled", at least if forcing is enabled (as is the
-; default).
-
-                   '((:executable-counterpart force))
-                   (theory 'executable-counterpart-minimal-theory)))
-  :doc
-  ":Doc-Section Theories
-
-  a minimal theory to enable~/~/
-
-  This ~ilc[theory] (~pl[theories]) enables only a few built-in
-  functions and executable counterparts.  It can be useful when you
-  want to formulate lemmas that rather immediately imply the theorem
-  to be proved, by way of a ~c[:use] hint (~pl[hints]), for example as
-  follows.
-  ~bv[]
-  :use (lemma-1 lemma-2 lemma-3)
-  :in-theory (union-theories '(f1 f2) (theory 'minimal-theory))
-  ~ev[]
-  In this example, we expect the current goal to follow from lemmas
-  ~c[lemma-1], ~c[lemma-2], and ~c[lemma-3] together with rules ~c[f1]
-  and ~c[f2] and some obvious facts about built-in functions (such as
-  the ~il[definition] of ~ilc[implies] and the
-  ~c[:]~ilc[executable-counterpart] of ~ilc[car]).  The
-  ~c[:]~ilc[in-theory] hint above is intended to speed up the proof by
-  turning off all inessential rules.~/
-
-  :cited-by theory-functions")
-
-(deftheory ground-zero (current-theory :here)
-
-; WARNING: Keep this near the end of *acl2-pass-2-files* in order for
-; the ground-zero theory to be as expected.
-
-  :doc
-  ":Doc-Section Theories
-
-  ~il[enable]d rules in the ~il[startup] theory~/
-
-  ACL2 concludes its initialization ~c[(boot-strapping)] procedure by
-  defining the theory ~c[ground-zero]; ~pl[theories].  In fact, this
-  theory is just the theory defined by ~c[(current-theory :here)] at
-  the conclusion of initialization; ~pl[current-theory].~/
-
-  Note that by evaluating the event
-  ~bv[]
-  (in-theory (current-theory 'ground-zero))
-  ~ev[]
-  you can restore the current theory to its value at the time you
-  started up ACL2.~/
-
-  :cited-by theory-functions")
-
-(deflabel
-
-; WARNING: Put this at the end of the last file in
-; *acl2-pass-2-files*.
-
-  end-of-pass-2)
