@@ -1,5 +1,5 @@
-; ACL2 Version 4.1 -- A Computational Logic for Applicative Common Lisp
-; Copyright (C) 2010  University of Texas at Austin
+; ACL2 Version 4.2 -- A Computational Logic for Applicative Common Lisp
+; Copyright (C) 2011  University of Texas at Austin
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
 ; (C) 1997 Computational Logic, Inc.  See the documentation topic NOTE-2-0.
@@ -609,9 +609,9 @@
                                                nil))))
                         ((if (ts-subsetp ts-lhs *ts-acl2-number*)
                              (or (ts-subsetp ts-rhs *ts-acl2-number*)
-                                 (not (ts-intersectp ts-lhs *ts-zero*)))
+                                 (ts-disjointp ts-lhs *ts-zero*))
                            (and (ts-subsetp ts-rhs *ts-acl2-number*)
-                                (not (ts-intersectp ts-rhs *ts-zero*))))
+                                (ts-disjointp ts-rhs *ts-zero*)))
 
 ; (implies (and (not (equal lhs rhs))
 ;               (or (and (acl2-numberp lhs)
